@@ -15,19 +15,16 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#[cfg(feature = "services-github")]
+/// Default scheme for github service.
+pub const GITHUB_SCHEME: &str = "github";
+
+mod backend;
+mod config;
 mod core;
-#[cfg(feature = "services-github")]
+mod deleter;
 mod error;
-#[cfg(feature = "services-github")]
 mod lister;
-#[cfg(feature = "services-github")]
 mod writer;
 
-#[cfg(feature = "services-github")]
-mod backend;
-#[cfg(feature = "services-github")]
 pub use backend::GithubBuilder as Github;
-
-mod config;
 pub use config::GithubConfig;
