@@ -15,52 +15,35 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use bb8::RunError;
-use bytes::Bytes;
+use std::path::PathBuf;
+use std::sync::Arc;
+use std::time::Duration;
+
 use http::Uri;
-use redis::cluster::ClusterClient;
-use redis::cluster::ClusterClientBuilder;
+use redis::Client;
 use redis::ConnectionAddr;
 use redis::ConnectionInfo;
 use redis::ProtocolVersion;
 use redis::RedisConnectionInfo;
-use redis::{AsyncCommands, Client};
-use std::fmt::Debug;
-use std::fmt::Formatter;
-use std::path::PathBuf;
-use std::time::Duration;
+use redis::cluster::ClusterClientBuilder;
 use tokio::sync::OnceCell;
 
+use super::REDIS_SCHEME;
+use super::config::RedisConfig;
 use super::core::*;
-use crate::raw::adapters::kv;
+use super::delete::RedisDeleter;
+use super::writer::RedisWriter;
 use crate::raw::*;
-use crate::services::RedisConfig;
 use crate::*;
 
 const DEFAULT_REDIS_ENDPOINT: &str = "tcp://127.0.0.1:6379";
 const DEFAULT_REDIS_PORT: u16 = 6379;
 
-impl Configurator for RedisConfig {
-    type Builder = RedisBuilder;
-    fn into_builder(self) -> Self::Builder {
-        RedisBuilder { config: self }
-    }
-}
-
 /// [Redis](https://redis.io/) services support.
 #[doc = include_str!("docs.md")]
-#[derive(Clone, Default)]
+#[derive(Debug, Default)]
 pub struct RedisBuilder {
-    config: RedisConfig,
-}
-
-impl Debug for RedisBuilder {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        let mut d = f.debug_struct("RedisBuilder");
-
-        d.field("config", &self.config);
-        d.finish_non_exhaustive()
-    }
+    pub(super) config: RedisConfig,
 }
 
 impl RedisBuilder {
@@ -141,10 +124,23 @@ impl RedisBuilder {
 
         self
     }
+
+    /// Sets the maximum number of connections managed by the pool.
+    ///
+    /// Defaults to 10.
+    ///
+    /// # Panics
+    ///
+    /// Will panic if `max_size` is 0.
+    #[must_use]
+    pub fn connection_pool_max_size(mut self, max_size: u32) -> Self {
+        assert!(max_size > 0, "max_size must be greater than zero!");
+        self.config.connection_pool_max_size = Some(max_size);
+        self
+    }
 }
 
 impl Builder for RedisBuilder {
-    const SCHEME: Scheme = Scheme::Redis;
     type Config = RedisConfig;
 
     fn build(self) -> Result<impl Access> {
@@ -172,12 +168,13 @@ impl Builder for RedisBuilder {
 
             let conn = OnceCell::new();
 
-            Ok(RedisBackend::new(Adapter {
+            Ok(RedisBackend::new(RedisCore {
                 addr: endpoints,
                 client: None,
                 cluster_client: Some(client),
                 conn,
                 default_ttl: self.config.default_ttl,
+                connection_pool_max_size: self.config.connection_pool_max_size,
             })
             .with_normalized_root(root))
         } else {
@@ -190,19 +187,20 @@ impl Builder for RedisBuilder {
             let client =
                 Client::open(self.get_connection_info(endpoint.clone())?).map_err(|e| {
                     Error::new(ErrorKind::ConfigInvalid, "invalid or unsupported scheme")
-                        .with_context("service", Scheme::Redis)
+                        .with_context("service", REDIS_SCHEME)
                         .with_context("endpoint", self.config.endpoint.as_ref().unwrap())
                         .with_context("db", self.config.db.to_string())
                         .set_source(e)
                 })?;
 
             let conn = OnceCell::new();
-            Ok(RedisBackend::new(Adapter {
+            Ok(RedisBackend::new(RedisCore {
                 addr: endpoint,
                 client: Some(client),
                 cluster_client: None,
                 conn,
                 default_ttl: self.config.default_ttl,
+                connection_pool_max_size: self.config.connection_pool_max_size,
             })
             .with_normalized_root(root))
         }
@@ -213,7 +211,7 @@ impl RedisBuilder {
     fn get_connection_info(&self, endpoint: String) -> Result<ConnectionInfo> {
         let ep_url = endpoint.parse::<Uri>().map_err(|e| {
             Error::new(ErrorKind::ConfigInvalid, "endpoint is invalid")
-                .with_context("service", Scheme::Redis)
+                .with_context("service", REDIS_SCHEME)
                 .with_context("endpoint", endpoint)
                 .set_source(e)
         })?;
@@ -247,9 +245,9 @@ impl RedisBuilder {
             Some(s) => {
                 return Err(
                     Error::new(ErrorKind::ConfigInvalid, "invalid or unsupported scheme")
-                        .with_context("service", Scheme::Redis)
+                        .with_context("service", REDIS_SCHEME)
                         .with_context("scheme", s),
-                )
+                );
             }
         };
 
@@ -267,112 +265,106 @@ impl RedisBuilder {
     }
 }
 
-/// Backend for redis services.
-pub type RedisBackend = kv::Backend<Adapter>;
-
-#[derive(Clone)]
-pub struct Adapter {
-    addr: String,
-    client: Option<Client>,
-    cluster_client: Option<ClusterClient>,
-    conn: OnceCell<bb8::Pool<RedisConnectionManager>>,
-
-    default_ttl: Option<Duration>,
+/// RedisBackend implements Access trait directly
+#[derive(Debug, Clone)]
+pub struct RedisBackend {
+    core: Arc<RedisCore>,
+    root: String,
+    info: Arc<AccessorInfo>,
 }
 
-// implement `Debug` manually, or password may be leaked.
-impl Debug for Adapter {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        let mut ds = f.debug_struct("Adapter");
+impl RedisBackend {
+    fn new(core: RedisCore) -> Self {
+        let info = AccessorInfo::default();
+        info.set_scheme(REDIS_SCHEME);
+        info.set_name(&core.addr);
+        info.set_root("/");
+        info.set_native_capability(Capability {
+            read: true,
+            write: true,
+            delete: true,
+            stat: true,
+            write_can_empty: true,
+            shared: true,
+            ..Default::default()
+        });
 
-        ds.field("addr", &self.addr);
-        ds.finish()
+        Self {
+            core: Arc::new(core),
+            root: "/".to_string(),
+            info: Arc::new(info),
+        }
+    }
+
+    fn with_normalized_root(mut self, root: String) -> Self {
+        self.info.set_root(&root);
+        self.root = root;
+        self
     }
 }
 
-impl Adapter {
-    async fn conn(&self) -> Result<bb8::PooledConnection<'_, RedisConnectionManager>> {
-        let pool = self
-            .conn
-            .get_or_try_init(|| async {
-                bb8::Pool::builder()
-                    .build(self.get_redis_connection_manager())
-                    .await
-                    .map_err(|err| {
-                        Error::new(ErrorKind::ConfigInvalid, "connect to redis failed")
-                            .set_source(err)
-                    })
-            })
-            .await?;
-        pool.get().await.map_err(|err| match err {
-            RunError::TimedOut => {
-                Error::new(ErrorKind::Unexpected, "get connection from pool failed").set_temporary()
-            }
-            RunError::User(err) => err,
-        })
+impl Access for RedisBackend {
+    type Reader = Buffer;
+    type Writer = RedisWriter;
+    type Lister = ();
+    type Deleter = oio::OneShotDeleter<RedisDeleter>;
+
+    fn info(&self) -> Arc<AccessorInfo> {
+        self.info.clone()
     }
 
-    fn get_redis_connection_manager(&self) -> RedisConnectionManager {
-        if let Some(_client) = self.client.clone() {
-            RedisConnectionManager {
-                client: self.client.clone(),
-                cluster_client: None,
-            }
+    async fn stat(&self, path: &str, _: OpStat) -> Result<RpStat> {
+        let p = build_abs_path(&self.root, path);
+
+        if p == build_abs_path(&self.root, "") {
+            Ok(RpStat::new(Metadata::new(EntryMode::DIR)))
         } else {
-            RedisConnectionManager {
-                client: None,
-                cluster_client: self.cluster_client.clone(),
+            let bs = self.core.get(&p).await?;
+            match bs {
+                Some(bs) => Ok(RpStat::new(
+                    Metadata::new(EntryMode::FILE).with_content_length(bs.len() as u64),
+                )),
+                None => Err(Error::new(ErrorKind::NotFound, "key not found in redis")),
             }
         }
     }
-}
 
-impl kv::Adapter for Adapter {
-    type Scanner = ();
+    async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
+        let p = build_abs_path(&self.root, path);
 
-    fn info(&self) -> kv::Info {
-        kv::Info::new(
-            Scheme::Redis,
-            self.addr.as_str(),
-            Capability {
-                read: true,
-                write: true,
-                shared: true,
-
-                ..Default::default()
-            },
-        )
-    }
-
-    async fn get(&self, key: &str) -> Result<Option<Buffer>> {
-        let mut conn = self.conn().await?;
-        let result: Option<Bytes> = conn.get(key).await.map_err(format_redis_error)?;
-        Ok(result.map(Buffer::from))
-    }
-
-    async fn set(&self, key: &str, value: Buffer) -> Result<()> {
-        let mut conn = self.conn().await?;
-        let value = value.to_vec();
-        if let Some(dur) = self.default_ttl {
-            let _: () = conn
-                .set_ex(key, value, dur.as_secs())
-                .await
-                .map_err(format_redis_error)?;
+        let range = args.range();
+        let buffer = if range.is_full() {
+            // Full read - use GET
+            match self.core.get(&p).await? {
+                Some(bs) => bs,
+                None => return Err(Error::new(ErrorKind::NotFound, "key not found in redis")),
+            }
         } else {
-            let _: () = conn.set(key, value).await.map_err(format_redis_error)?;
-        }
-        Ok(())
+            // Range read - use GETRANGE
+            let start = range.offset() as isize;
+            let end = match range.size() {
+                Some(size) => (range.offset() + size - 1) as isize,
+                None => -1, // Redis uses -1 for end of string
+            };
+
+            match self.core.get_range(&p, start, end).await? {
+                Some(bs) => bs,
+                None => return Err(Error::new(ErrorKind::NotFound, "key not found in redis")),
+            }
+        };
+
+        Ok((RpRead::new(), buffer))
     }
 
-    async fn delete(&self, key: &str) -> Result<()> {
-        let mut conn = self.conn().await?;
-        let _: () = conn.del(key).await.map_err(format_redis_error)?;
-        Ok(())
+    async fn write(&self, path: &str, _: OpWrite) -> Result<(RpWrite, Self::Writer)> {
+        let p = build_abs_path(&self.root, path);
+        Ok((RpWrite::new(), RedisWriter::new(self.core.clone(), p)))
     }
 
-    async fn append(&self, key: &str, value: &[u8]) -> Result<()> {
-        let mut conn = self.conn().await?;
-        let _: () = conn.append(key, value).await.map_err(format_redis_error)?;
-        Ok(())
+    async fn delete(&self) -> Result<(RpDelete, Self::Deleter)> {
+        Ok((
+            RpDelete::default(),
+            oio::OneShotDeleter::new(RedisDeleter::new(self.core.clone(), self.root.clone())),
+        ))
     }
 }
